# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json


class CommandError(Exception):
    pass


class ClientException(Exception):
    """The base exception class for all exceptions this library raises."""
    message = 'Unknown Error'

    # noinspection PyMissingConstructor
    def __init__(self, code, message=None, request_id=None,
                 url=None, method=None):
        self.code = code
        self.message = message or self.__class__.message
        self.request_id = request_id
        self.url = url
        self.method = method

    def __str__(self):
        formatted_string = '%s (HTTP %s)' % (self.message, self.code)
        if self.request_id:
            formatted_string += ' (Request-ID: %s)' % self.request_id

        return formatted_string


def from_response(resp, url, method):
    msg = None
    if resp.text:
        try:
            body = json.loads(resp.text)
            msg = body.get('description', None)
        except Exception as e:
            print('get msg failed, resp.text:%s, e:%s ' % (resp.text, e))

    return ClientException(resp.status_code, message=msg,
                           url=url, method=method)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class Topology(object):
    URL = 'v1/topology/'

    def __init__(self, api):
        self.api = api

    def get(self,
            limit=None,
            graph_type='graph',
            query=None,
            root=None,
            all_tenants=False):
        """Get a topology

        :param root:  the root of the topology graph
        :param query: the query filter for the topology
        :param graph_type: graph can be tree or graph
        :param limit: the depth of the topology graph
        :param all_tenants: show entities in graph of all tenants
        """

        params = dict(depth=limit,
                      graph_type=graph_type,
                      query=query,
                      root=root,
                      all_tenants=all_tenants)
        return self.api.post(self.URL, json=params).json()



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import requests

from keystoneauth1 import loading
from keystoneauth1 import plugin
from oslo_log import log


LOG = log.getLogger(__name__)


# noinspection PyAbstractClass
class VitrageNoAuthPlugin(plugin.BaseAuthPlugin):
    """No authentication plugin for Vitrage

    This is a keystoneauth plugin that instead of
    doing authentication, it just fill the 'x-user-id'
    and 'x-project-id' headers with the user provided one.
    """
    def __init__(self, user_id, project_id, roles, endpoint):
        self._user_id = user_id
        self._project_id = project_id
        self._endpoint = endpoint
        self._roles = roles

    def get_token(self, session, **kwargs):
        return '<no-token-needed>'

    def get_headers(self, session, **kwargs):
        return {'x-user-id': self._user_id,
                'x-project-id': self._project_id,
                'x-roles': self._roles}

    def get_user_id(self, session, **kwargs):
        return self._user_id

    def get_project_id(self, session, **kwargs):
        return self._project_id

    def get_endpoint(self, session, **kwargs):
        return self._endpoint


class VitrageOpt(loading.Opt):
    @property
    def argparse_args(self):
        return ['--%s' % o.name for o in self._all_opts]

    @property
    def argparse_default(self):
        # select the first ENV that is not false-y or return None
        for o in self._all_opts:
            v = os.environ.get('VITRAGE_%s' % o.name.replace('-', '_').upper())
            if v:
                return v
        return self.default


class VitrageNoAuthLoader(loading.BaseLoader):
    plugin_class = VitrageNoAuthPlugin

    def get_options(self):
        options = super(VitrageNoAuthLoader, self).get_options()
        options.extend([
            VitrageOpt('user-id', help='User ID', required=True),
            VitrageOpt('project-id', help='Project ID', required=True),
            VitrageOpt('roles', help='Roles', default="admin"),
            VitrageOpt('endpoint', help='Vitrage endpoint', required=True),
        ])
        return options


# noinspection PyAbstractClass
class VitrageKeycloakPlugin(plugin.BaseAuthPlugin):
    """Authentication plugin for Keycloak """

    def __init__(self, username, password, realm_name, endpoint, auth_url,
                 openid_client_id):
        self.username = username
        self.password = password
        self.realm_name = realm_name
        self.endpoint = endpoint
        self.auth_url = auth_url
        self.client_id = openid_client_id
        self.verify = True

    def get_headers(self, session, **kwargs):
        self.verify = session.verify
        return {'X-Auth-Token': self._authenticate_keycloak(),
                'x-user-id': self.username,
                'x-project-id': self.realm_name}

    def get_endpoint(self, session, **kwargs):
        return self.endpoint

    def _authenticate_keycloak(self):
        keycloak_endpoint = "%s/realms/%s/protocol/openid-connect/token" % \
                            (self.auth_url, self.realm_name)

        body = {
            'grant_type': 'password',
            'username': self.username,
            'password': self.password,
            'client_id': self.client_id,
            'scope': 'profile'
        }

        resp = requests.post(keycloak_endpoint,
                             data=body,
                             verify=self.verify)

        try:
            resp.raise_for_status()
        except Exception as e:
            LOG.error('Failed to get access token: %s', str(e))

        return resp.json()['access_token']


class VitrageKeycloakLoader(loading.BaseLoader):
    plugin_class = VitrageKeycloakPlugin

    def get_options(self):
        options = super(VitrageKeycloakLoader, self).get_options()
        options.extend([
            VitrageOpt('username', help='User Name', required=True),
            VitrageOpt('password', help='password', required=True),
            VitrageOpt('realm-name', help='Realm Name', required=True),
            VitrageOpt('endpoint', help='Vitrage Endpoint', required=True),
            VitrageOpt('auth-url', help='Keycloak Url', required=True),
            VitrageOpt('openid-client-id', help='Keycloak client id',
                       required=True),
        ])
        return options



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from cliff import lister
from cliff import show

from vitrageclient.common import utils


# noinspection PyAbstractClass
class WebhookShow(show.ShowOne):
    """Show a webhook """

    def get_parser(self, prog_name):
        parser = super(WebhookShow, self).get_parser(prog_name)
        parser.add_argument('id', help='id of webhook to show')
        return parser

    def take_action(self, parsed_args):
        id = parsed_args.id
        webhook = utils.get_client(self).webhook.show(id=id)
        return self.dict2columns(webhook)


class WebhookList(lister.Lister):
    """List all webhooks in the database"""

    POST_PROPS = \
        (
            ('ID', 'id'),
            ('Created At', 'created_at'),
            ('Project ID', 'project_id'),
            ('URL', 'url'),
            ('Headers', 'headers'),
            ('Filter', 'regex_filter')
        )

    def get_parser(self, prog_name):
        parser = super(WebhookList, self).get_parser(prog_name)
        parser.add_argument('--all-tenants',
                            default=False,
                            dest='all_tenants',
                            action='store_true',
                            help='Shows webhooks of all the tenants')
        return parser

    def take_action(self, parsed_args):
        all_tenants = parsed_args.all_tenants
        webhooks = utils.get_client(self).webhook.list(all_tenants=all_tenants)

        return utils.list2cols_with_rename(self.POST_PROPS, webhooks)


class WebhookAdd(show.ShowOne):
    """Add a new webhook to the database"""
    def get_parser(self, prog_name):
        parser = super(WebhookAdd, self).get_parser(prog_name)
        parser.add_argument('--url',
                            required=True,
                            help='url to post to'
                            )
        parser.add_argument('--regex_filter',
                            required=False,
                            help='a regular expression json specifying alarm '
                                 'filters'
                            )
        parser.add_argument('--headers',
                            required=False,
                            help='json to be included in the request header'
                            )

        return parser

    def take_action(self, parsed_args):
        result = utils.get_client(self).webhook.add(
            url=parsed_args.url,
            regex_filter=parsed_args.regex_filter,
            headers=parsed_args.headers)

        return self.dict2columns(result)


class WebhookDelete(show.ShowOne):
    """Delete a webhook """

    def get_parser(self, prog_name):
        parser = super(WebhookDelete, self).get_parser(prog_name)
        parser.add_argument('id', help='id of webhook to delete')
        return parser

    def take_action(self, parsed_args):
        id = parsed_args.id
        result = utils.get_client(self).webhook.delete(id=id)
        return self.dict2columns(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class Rca(object):
    url = 'v1/rca/'

    def __init__(self, api):
        self.api = api

    def get(self, alarm_id, all_tenants=False):
        """Get RCA for an alarm

        :param all_tenants: should return all tenants rca
        :param alarm_id: the id of the alarm
        """
        params = dict(alarm_id=alarm_id,
                      all_tenants=all_tenants)
        return self.api.get(self.url, params=params).json()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrageclient import exceptions as exc

from keystoneauth1 import adapter as keystoneauth
from oslo_utils import importutils

profiler_web = importutils.try_import('osprofiler.web')


# noinspection PyPep8Naming
def Client(version, *args, **kwargs):
    module = importutils.import_versioned_module('vitrageclient',
                                                 version, 'client')
    client_class = getattr(module, 'Client')
    return client_class(*args, **kwargs)


class VitrageClient(keystoneauth.Adapter):
    def request(self, url, method, **kwargs):
        headers = kwargs.setdefault('headers', {})
        headers.setdefault('Accept', 'application/json')

        if profiler_web:
            # no header will be added if profiler is not initialized
            headers.update(profiler_web.get_trace_id_headers())

        raise_exc = kwargs.pop('raise_exc', True)
        resp = super(VitrageClient, self).request(url, method,
                                                  raise_exc=False,
                                                  **kwargs)

        if raise_exc and resp.status_code >= 400:
            raise exc.from_response(resp, url, method)
        return resp



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import logging

from vitrageclient import client

from vitrageclient.v1 import alarm
from vitrageclient.v1 import event
from vitrageclient.v1 import healthcheck
from vitrageclient.v1 import rca
from vitrageclient.v1 import resource
from vitrageclient.v1 import service
from vitrageclient.v1 import status
from vitrageclient.v1 import template
from vitrageclient.v1 import topology
from vitrageclient.v1 import webhook


class Client(object):

    def __init__(self, session=None, service_type='rca', **kwargs):
        logger = logging.getLogger(__name__)
        self._api = client.VitrageClient(session, service_type=service_type,
                                         logger=logger, **kwargs)
        self.topology = topology.Topology(self._api)
        self.resource = resource.Resource(self._api)
        self.alarm = alarm.Alarm(self._api)
        self.rca = rca.Rca(self._api)
        self.template = template.Template(self._api)
        self.event = event.Event(self._api)
        self.healthcheck = healthcheck.HealthCheck(self._api)
        self.webhook = webhook.Webhook(self._api)
        self.service = service.Service(self._api)
        self.status = status.Status(self._api)



# Copyright 2017 - Nokia Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class Event(object):
    URL = 'v1/event/'

    def __init__(self, api):
        self.api = api

    def post(self, event_time, event_type, details):
        """Post an event """

        params = dict(time=event_time,
                      type=event_type,
                      details=details)
        self.api.post(self.URL, json=params)



# Copyright 2017 - Nokia Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import argparse
from cliff import command
from datetime import datetime
from iso8601 import iso8601
from iso8601 import ParseError


# noinspection PyAbstractClass
class EventPost(command.Command):
    """Post an event to Vitrage"""

    @staticmethod
    def iso8601(argument_value):
        try:
            if argument_value:
                iso8601.parse_date(argument_value)
        except ParseError:
            msg = "%s must be an iso8601 date" % argument_value
            raise argparse.ArgumentTypeError(msg)

    def get_parser(self, prog_name):
        parser = super(EventPost, self).get_parser(prog_name)

        parser.add_argument('--type',
                            required=True,
                            help='The type of the event')

        parser.add_argument('--time',
                            default='',
                            type=self.iso8601,
                            help='The timestamp of the event in ISO 8601 '
                                 'format: YYYY-MM-DDTHH:MM:SS.mmmmmm. '
                                 'If not specified, the current time is used')

        parser.add_argument('--details',
                            default='{}',
                            help='A json string with the event details')

        return parser

    def take_action(self, parsed_args):

        if parsed_args.time:
            event_time = parsed_args.time
        else:
            event_time = datetime.now().isoformat()

        event_type = parsed_args.type
        details = parsed_args.details

        self.app.client.event.post(event_time=event_time,
                                   event_type=event_type,
                                   details=json.loads(details))



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse

from oslotest import base


class CliTestCase(base.BaseTestCase):

    """Test case base class for all unit tests."""

    # original error method of argparse uses exit
    # I just want to raise an exception and get the error message
    # that exit outputs
    @staticmethod
    def _my_parser_error_func(message):
        raise argparse.ArgumentTypeError(message)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class Resource(object):
    url = 'v1/resources/'

    def __init__(self, api):
        self.api = api

    def list(self, resource_type=None, all_tenants=False, query=None):
        """Get a all resources

        :param all_tenants: should return all tenants resources
        :param resource_type: the type for the resources
        :param query: the query filter for the vertices
        """
        params = dict(resource_type=resource_type, all_tenants=all_tenants,
                      query=query)
        return self.api.post(self.url, json=params).json()

    def get(self, vitrage_id):
        """Get a resource

        :param vitrage_id: the vitrage_id of the resource
        """
        url = self.url + vitrage_id
        return self.api.get(url).json()

    def count(self, resource_type=None, all_tenants=False, query=None,
              group_by=None):
        """Get a count of all resources

        :param all_tenants: should return all tenants resources
        :param resource_type: the type for the resources
        :param query: the query filter for the vertices
        :param group_by: a property name to group by it's values
        """
        params = dict(resource_type=resource_type, all_tenants=all_tenants,
                      query=query, group_by=group_by)
        return self.api.post(self.url + 'count/', json=params).json()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from cliff import lister
from cliff import show

from vitrageclient.common import utils
from vitrageclient import exceptions as exc


class AlarmList(lister.Lister):
    """List all alarms"""

    def get_parser(self, prog_name):
        parser = super(AlarmList, self).get_parser(prog_name)
        parser.add_argument("vitrage_id",
                            default='all',
                            nargs='?',
                            metavar="<vitrage id>",
                            help="Vitrage id of the affected resource")

        parser.add_argument('--all-tenants',
                            default=False,
                            dest='all_tenants',
                            action='store_true',
                            help='Shows alarms of all the tenants in the '
                                 'entity graph')

        parser.add_argument('--limit',
                            dest='limit',
                            help='Maximal number of alarms to show. Default '
                                 'is 1000')

        parser.add_argument('--marker',
                            dest='marker',
                            help='Marker for the next page')

        return parser

    def take_action(self, parsed_args):
        vitrage_id = parsed_args.vitrage_id
        all_tenants = parsed_args.all_tenants
        limit = parsed_args.limit
        marker = parsed_args.marker

        alarms = utils.get_client(self).alarm.list(vitrage_id=vitrage_id,
                                                   limit=limit,
                                                   marker=marker,
                                                   all_tenants=all_tenants)
        return utils.list2cols_with_rename(
            (
                ('ID', 'vitrage_id'),
                ('Type', 'vitrage_type'),
                ('Name', 'name'),
                ('Resource Type', 'vitrage_resource_type'),
                ('Resource ID', 'vitrage_resource_id'),
                ('Severity', 'vitrage_operational_severity'),
                ('Update Time', 'update_timestamp'),
            ), alarms)


class AlarmHistory(lister.Lister):
    """List the alarm history"""

    def get_parser(self, prog_name):
        parser = super(AlarmHistory, self).get_parser(prog_name)
        parser.add_argument('--all-tenants',
                            default=False,
                            dest='all_tenants',
                            action='store_true',
                            help='Shows alarms of all the tenants in the '
                                 'entity graph')

        parser.add_argument('--limit',
                            dest='limit',
                            help='Maximal number of alarms to show. Default '
                                 'is 1000')

        parser.add_argument('--marker',
                            dest='marker',
                            help='Marker for the next page')

        parser.add_argument('--start',
                            dest='start',
                            help='list alarm from this date')

        parser.add_argument('--end',
                            dest='end',
                            help='list alarm until this date')

        return parser

    def take_action(self, parsed_args):
        all_tenants = parsed_args.all_tenants
        limit = parsed_args.limit
        marker = parsed_args.marker
        start = parsed_args.start
        end = parsed_args.end

        if end and not start:
            raise exc.CommandError("--end argument must be used with --start")

        alarms = utils.get_client(self).alarm.history(limit=limit,
                                                      marker=marker,
                                                      start=start,
                                                      end=end,
                                                      all_tenants=all_tenants)
        return utils.list2cols_with_rename(
            (
                ('ID', 'vitrage_id'),
                ('Type', 'vitrage_type'),
                ('Name', 'name'),
                ('Resource Type', 'vitrage_resource_type'),
                ('Resource ID', 'vitrage_resource_id'),
                ('Severity', 'vitrage_operational_severity'),
                ('Start Time', 'start_timestamp'),
                ('End Time', 'end_timestamp'),
            ), alarms)


class AlarmShow(show.ShowOne):
    """Show an alarm"""

    def get_parser(self, prog_name):
        parser = super(AlarmShow, self).get_parser(prog_name)
        parser.add_argument("vitrage_id", help="Vitrage id of the alarm")
        return parser

    def take_action(self, parsed_args):
        vitrage_id = parsed_args.vitrage_id
        alarm = utils.get_client(self).alarm.get(vitrage_id=vitrage_id)
        return self.dict2columns(alarm)


# noinspection PyAbstractClass
class AlarmCount(show.ShowOne):
    """Show a count of all alarms"""

    def get_parser(self, prog_name):
        parser = super(AlarmCount, self).get_parser(prog_name)
        parser.add_argument('--all-tenants',
                            default=False,
                            dest='all_tenants',
                            action='store_true',
                            help='Shows counts for alarms of all '
                                 'the tenants')

        return parser

    @property
    def formatter_default(self):
        return 'json'

    def take_action(self, parsed_args):
        all_tenants = parsed_args.all_tenants

        counts = utils.get_client(self).alarm.count(
            all_tenants=all_tenants)

        return self.dict2columns(counts)



# Copyright 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from osc_lib import utils


DEFAULT_RCA_API_VERSION = '1'
API_VERSION_OPTION = 'os_rca_api_version'
API_NAME = "rca"
API_VERSIONS = {
    "1": "vitrageclient.v1.client.Client",
}


def make_client(instance):
    """Returns a vitrage service client."""
    version = instance._api_version[API_NAME]
    try:
        version = int(version)
    except ValueError:
        version = float(version)

    vitrage_client = utils.get_client_class(
        API_NAME,
        version,
        API_VERSIONS)
    instance.setup_auth()
    return vitrage_client(session=instance.session,
                          interface=instance.interface,
                          region_name=instance.region_name)


def build_option_parser(parser):
    """Hook to add global options."""
    parser.add_argument(
        '--os-rca-api-version',
        metavar='<rca-api-version>',
        default=utils.env(
            'OS_RCA_API_VERSION',
            default=DEFAULT_RCA_API_VERSION),
        help=('Rca API version, default=' +
              DEFAULT_RCA_API_VERSION +
              ' (Env: OS_RCS_API_VERSION)'))
    return parser



# Copyright 2016 - Nokia Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from vitrageclient.common import yaml_utils
from vitrageclient import exceptions as exc


class Template(object):

    url = 'v1/template/'

    def __init__(self, api):
        self.api = api

    def list(self):
        """Get templates list"""
        return self.api.get(self.url).json()

    def versions(self):
        """Get templates versions"""
        return self.api.get(self.url + 'versions').json()

    def show(self, _id):
        """Show template content"""

        url = self.url + _id
        return self.api.get(url).json()

    def add(self, path=None, template_type=None,
            params=None, template_str=None, overwrite=False):
        """Add a new template

        :param path: (optional) The template file path or templates dir path
        :param template_type: (optional) The template type, in case it is not
        written inside the template metadata section
        :param params: (optional) Actual values for the template parameters
        :param template_str: (optional) A string representation of the template
        :param overwrite: (optional) overwrite the template if exists
        yaml
        Either path or template_str must exist (but not both)

        :return:
        """
        files_content = \
            self._load_template(path=path, template_str=template_str)
        api_params = dict(templates=files_content,
                          template_type=template_type,
                          params=params, overwrite=overwrite)
        return self.api.put(self.url, json=api_params).json()

    def delete(self, ids):
        """Delete existing"""
        params = dict(id=ids)
        return self.api.delete(self.url, json=params).json()

    def validate(self, path=None, template_type=None,
                 params=None, template_str=None):
        """Template validation

        Make sure that the template file is correct in terms of syntax
        and content.
        It is possible to pass a specific file path in order to validate one
        template, or directory path for validation of several templates (the
        directory must contain only templates)

        :param path: (optional) The template file path or templates dir path
        :param template_type: (optional) The template type, in case it is not
        written inside the template metadata section
        :param params: (optional) Actual values for the template parameters
        :param template_str: (optional) A string representation of the template
        yaml
        Either path or template_str must exist (but not both)

        :return:
        """
        files_content = \
            self._load_template(path=path, template_str=template_str)
        api_params = dict(templates=files_content,
                          template_type=template_type,
                          params=params)
        return self.api.post(self.url, json=api_params).json()

    @classmethod
    def _load_yaml_files(cls, path):
        if os.path.isdir(path):

            files_content = []
            for file_name in os.listdir(path):

                file_path = '%s/%s' % (path, file_name)
                if os.path.isfile(file_path):
                    template = cls._load_yaml_file(file_path)
                    files_content.append((file_path, template))
        else:
            files_content = [(path, cls._load_yaml_file(path))]

        return files_content

    @classmethod
    def _load_yaml_file(cls, path):
        with open(path, 'r') as stream:
            return cls._load_yaml(stream)

    @classmethod
    def _load_yaml(cls, yaml_content):
        try:
            return yaml_utils.load(yaml_content)
        except ValueError as e:
            message = 'Could not load template: %s. Reason: %s' \
                      % (yaml_content, e)
            raise exc.CommandError(message)

    @classmethod
    def _load_template(cls, path, template_str):
        if path:
            files_content = cls._load_yaml_files(path)
        elif template_str:
            files_content = [(None, cls._load_yaml(template_str))]
        else:
            raise exc.CommandError(
                'Add template API must be called with either \'path\' or '
                '\'template_str\'')

        return files_content
